/**
 * Copyright (c) 2016 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import UIKit

class FileProvider: NSFileProviderExtension {

  // MARK: - Properties
  var fileCoordinator: NSFileCoordinator {
    let fileCoordinator = NSFileCoordinator()
    fileCoordinator.purposeIdentifier = self.providerIdentifier
    return fileCoordinator
  }

  // MARK: - Initializers
  override init() {
    super.init()

    fileCoordinator.coordinate(writingItemAt: documentStorageURL, options: NSFileCoordinator.WritingOptions(), error: nil, byAccessor: { newURL in
      // ensure the documentStorageURL actually exists
      do {
        try FileManager.default.createDirectory(at: newURL, withIntermediateDirectories: true, attributes: nil)
      } catch {
        // Handle error
      }
    })
  }
  
  
  // MARK: - Overridden Instance Methods
  override func providePlaceholder(at url: URL, completionHandler: @escaping (Error?) -> ()) {
    // Should call writePlaceholderAtURL(_:withMetadata:error:) with the placeholder URL, then call the completion handler with the error if applicable.
    let fileName = url.lastPathComponent

    let placeholderURL = NSFileProviderExtension.placeholderURL(for: self.documentStorageURL.appendingPathComponent(fileName))
    // TODO: get file size for file at <url> from model
    let fileSize = 0
    let metadata = [URLResourceKey.fileSizeKey: fileSize]
    do {
      try NSFileProviderExtension.writePlaceholder(at: placeholderURL, withMetadata: metadata)
    } catch {
      // Handle error
    }

    completionHandler(nil)
  }
  
  override func startProvidingItem(at url: URL, completionHandler: @escaping (Error?) -> ()) {
    guard let fileData = try? Data(contentsOf: url) else {
      // NOTE: you would generate an NSError to supply to the completionHandler
      // here however that is outside of the scope for this tutorial
      completionHandler(nil)
      return
    }

    do {
      _ = try fileData.write(to: url, options: NSData.WritingOptions())
      completionHandler(nil)
    } catch let error as NSError {
      print("error writing file to URL")
      completionHandler(error)
    }
  }

  override func itemChanged(at url: URL) {
    // Called at some point after the file has changed; the provider may then trigger an upload
    // TODO: mark file at <url> as needing an update in the model; kick off update process
    print("Item changed at URL: \(url)")
  }

  override func stopProvidingItem(at url: URL) {
    // Called after the last claim to the file has been released. At this point, it is safe for the file provider to remove the content file.
    // Care should be taken that the corresponding placeholder file stays behind after the content file has been deleted.
    do {
      _ = try FileManager.default.removeItem(at: url)
    } catch {
      // Handle error
    }

    providePlaceholder(at: url, completionHandler: { error in
      // TODO: handle any error, do any necessary cleanup
    })
  }
}
